#include "cross_correl_fft.h"

void dummy_proc_fe(
  bool direction,
  config_t* config,
  cmpxDataIn in[FFT_LENGTH],
  cmpxDataIn out[FFT_LENGTH])
{
  int i;

  config->setDir(direction);
  config->setSch(0x2AB);

  for (i=0; i<FFT_LENGTH; i++)
  {
    out[i] = in[i];
  }
}

void dummy_proc_be(
  status_t* status_in,
  bool* ovflo,
  cmpxDataOut in[FFT_LENGTH],
  cmpxDataOut out[FFT_LENGTH])
{
    int i;

    for (i=0; i<FFT_LENGTH; i++)
    {
      out[i] = in[i];
    }

    *ovflo = status_in->getOvflo() & 0x1;
}

void fft(
  bool direction,
  complex<data_in_t> in[FFT_LENGTH],
  complex<data_out_t> out[FFT_LENGTH],
  bool* ovflo)
{
  #pragma HLS interface ap_hs port=direction
  #pragma HLS interface ap_fifo depth=1 port=ovflo
  #pragma HLS interface ap_fifo depth=FFT_LENGTH port=in,out
  #pragma HLS aggregate variable=in compact=bit
  #pragma HLS aggregate variable=out compact=bit
  #pragma HLS dataflow

  complex<data_in_t> xn[FFT_LENGTH];
  complex<data_out_t> xk[FFT_LENGTH];
  config_t fft_config;
  status_t fft_status;

  dummy_proc_fe(direction, &fft_config, in, xn);
  hls::fft<config1>(xn, xk, &fft_status, &fft_config);
  dummy_proc_be(&fft_status, ovflo, xk, out);
}

int32_t correlate(volatile int16_t *a, volatile int16_t *b, int32_t length)
{
  #pragma HLS INTERFACE m_axi port=a depth=FFT_LENGTH offset=slave
  #pragma HLS INTERFACE m_axi port=b depth=FFT_LENGTH offset=slave
  #pragma HLS INTERFACE s_axilite port=length
  #pragma HLS INTERFACE s_axilite port=return

  int32_t i=0;
  int32_t idx=0;
  double max=0;

  cmpxDataIn  fft_in_a[FFT_LENGTH];
  cmpxDataOut fft_out_a[FFT_LENGTH];
  cmpxDataIn  fft_in_b[FFT_LENGTH];
  cmpxDataOut fft_out_b[FFT_LENGTH];
  cmpxDataOut fft_mul[FFT_LENGTH];
  cmpxDataOut ifft_mul[FFT_LENGTH];

  LOOP1:
  for (i=0; i<FFT_LENGTH; i++) {
    if(i<length) {
      fft_in_a[i] = cmpxDataIn((double)a[i]/FFT_VALUE_MAX, 0);
      fft_in_b[i] = cmpxDataIn((double)b[i]/FFT_VALUE_MAX, 0);
    }
    else
    {
      fft_in_a[i] = cmpxDataIn(0, 0);
      fft_in_b[i] = cmpxDataIn(0, 0);
    }
  }

  bool ovflo_a = false;
  bool ovflo_b = false;
  bool ovflo_c = false;

  fft(true, fft_in_a, fft_out_a, &ovflo_a);
  fft(true, fft_in_b, fft_out_b, &ovflo_b);

  for (i=0; i<FFT_LENGTH; i++)
  {
    if(i<length)
    {
      fft_mul[i] = fft_out_a[i] * fft_out_b[i];
    }
    else
    {
      fft_mul[i] = 0;
    }
  }

  fft(false, fft_mul, ifft_mul, &ovflo_c);

  for (i=0; i<FFT_LENGTH; i++)
  {
    if (i<length)
    {
      if (ifft_mul[i].real()>max)
      {
        max = ifft_mul[i].real();
        idx = i;
      }
    }
    else
    {
      break;
    }
  }

  return idx;
}
