#!/usr/local/bin/python3
# -*- coding: utf-8 -*-

import sys
import os
import numpy as np
import operator

import time
import threading

from pynq import Overlay
from pynq import DefaultIP
from pynq import allocate


class Correlation_Driver_0(DefaultIP):
    bindto = ['ven:lib:correlate:1.1']
    buf_a = 0
    buf_b = 0

    def __init__(self, description):
        super().__init__(description=description)
        self.buf_a = allocate(shape=(1024,), dtype=np.int16)
        self.buf_b = allocate(shape=(1024,), dtype=np.int16)

    def correlate(self, a, b, data_len):
        time_1 = time.time()
        self.buf_a[0:data_len] = a[0:data_len]
        self.buf_b[0:data_len] = b[0:data_len]
        time_2 = time.time()
        self.register_map.a_1 = self.buf_a.physical_address
        self.register_map.b_1 = self.buf_b.physical_address
        self.register_map.length_r = data_len
        self.register_map.CTRL.AP_START = 1
        time_3 = time.time()
        while self.register_map.CTRL.AP_DONE != 1:
            time.sleep(0.000001)
        time_4 = time.time()
        return time_4 - time_1, time_2 - time_1, time_3 - time_2, time_4 - time_3

ol = Overlay("./bitstream/correlation/correlation.bit")
correl_ip_0 = ol.correlate_0
correl_ip_1 = ol.correlate_1
correl_ip_2 = ol.correlate_2
correl_ip_3 = ol.correlate_3

def cross_correlation(a, b, len, loop_num, th_idx):
    if th_idx == 0:
        fpga_ip = correl_ip_0
    elif th_idx == 1:
        fpga_ip = correl_ip_1
    elif th_idx == 2:
        fpga_ip = correl_ip_2
    elif th_idx == 3:
        fpga_ip = correl_ip_3
    else:
        fpga_ip = correl_ip_0

    total_s = 0
    copy_s = 0
    set_cmd_s = 0
    run_s = 0

    time_start = time.time()
    print("[RunTime] Thread(%d), Start, timestamp: %f" % (th_idx, time_start))
    for i in range(loop_num):
        total, copy, set_cmd, run = fpga_ip.correlate(a, b, len)
        total_s = total_s + total
        copy_s = copy_s + copy
        set_cmd_s = set_cmd_s + set_cmd
        run_s = run_s + run
    time_finish = time.time()
    print("[RunTime] Thread(%d), Finish, %f sec" % (th_idx, time_finish-time_start))
    print("[RunTime] Thread(%d), Average[total, copy, set_cmd, run]: [%f, %f, %f, %f]" % (th_idx, total_s/loop_num, copy_s/loop_num, set_cmd_s/loop_num, run_s/loop_num))

def test():
    thread_list = []
    thread_num = 4
    data_len = 1024
    loop_num = 35000
    data_a = np.random.randint(32767, size=data_len)
    data_b = np.random.randint(32767, size=data_len)

    for th in range(thread_num):
      print("###########################")
      print("[Info] thread_num: %s, data_len: %d, loop_num: %d" % (th+1, data_len, loop_num))
      print("[RunTime] Begin")
      time_begin = time.time()
      for i in range(th+1):
          thread_list.append(threading.Thread(target=cross_correlation, args=(data_a, data_b, data_len, loop_num, i,)))
      for t in thread_list:
          t.start()
      for t in thread_list:
          t.join()
      time_end = time.time()
      print("[RunTime] End, %f sec" % (time_end-time_begin))
      thread_list.clear()
    print("###########################")

if __name__ == '__main__':
        test()
